@@ -25,6 +25,7 @@ import (
 	"github.com/spf13/cobra"
 
 	"github.com/GoogleContainerTools/skaffold/pkg/skaffold/output"
+	"github.com/GoogleContainerTools/skaffold/pkg/skaffold/parser"
 	"github.com/GoogleContainerTools/skaffold/pkg/skaffold/schema"
 	latestV1 "github.com/GoogleContainerTools/skaffold/pkg/skaffold/schema/latest/v1"
 	"github.com/GoogleContainerTools/skaffold/pkg/skaffold/schema/validation"
@@ -81,11 +82,14 @@ func fix(out io.Writer, configFile string, toVersion string, overwrite bool) err
 	// TODO(dgageot): We should be able run validations on any schema version
 	// but that's not the case. They can only run on the latest version for now.
 	if toVersion == latestV1.Version {
-		var cfgs []*latestV1.SkaffoldConfig
+		var cfgs parser.SkaffoldConfigSet
 		for _, cfg := range versionedCfgs {
-			cfgs = append(cfgs, cfg.(*latestV1.SkaffoldConfig))
+			cfgs = append(cfgs, &parser.SkaffoldConfigEntry{
+				SkaffoldConfig: cfg.(*latestV1.SkaffoldConfig),
+				SourceFile:     configFile,
+				IsRootConfig:   true})
 		}
-		if err := validation.Process(cfgs); err != nil {
+		if err := validation.Process(cfgs, validation.GetValidationOpts(opts)); err != nil {
 			return fmt.Errorf("validating upgraded config: %w", err)
 		}
 	}
