@@ -17,7 +17,6 @@ limitations under the License.
 package logger
 
 import (
-	"bufio"
 	"context"
 	"fmt"
 	"io"
@@ -28,10 +27,10 @@ import (
 	"github.com/sirupsen/logrus"
 	v1 "k8s.io/api/core/v1"
 
-	eventV2 "github.com/GoogleContainerTools/skaffold/pkg/skaffold/event/v2"
 	"github.com/GoogleContainerTools/skaffold/pkg/skaffold/graph"
 	"github.com/GoogleContainerTools/skaffold/pkg/skaffold/kubectl"
 	"github.com/GoogleContainerTools/skaffold/pkg/skaffold/kubernetes"
+	"github.com/GoogleContainerTools/skaffold/pkg/skaffold/log/stream"
 	"github.com/GoogleContainerTools/skaffold/pkg/skaffold/output"
 	latestV1 "github.com/GoogleContainerTools/skaffold/pkg/skaffold/schema/latest/v1"
 	"github.com/GoogleContainerTools/skaffold/pkg/skaffold/tag"
@@ -52,6 +51,7 @@ type LogAggregator struct {
 	events            chan kubernetes.PodEvent
 	trackedContainers trackedContainers
 	outputLock        sync.Mutex
+	namespaces        *[]string
 }
 
 type Config interface {
@@ -61,7 +61,7 @@ type Config interface {
 }
 
 // NewLogAggregator creates a new LogAggregator for a given output.
-func NewLogAggregator(cli *kubectl.CLI, podSelector kubernetes.PodSelector, config Config) *LogAggregator {
+func NewLogAggregator(cli *kubectl.CLI, podSelector kubernetes.PodSelector, namespaces *[]string, config Config) *LogAggregator {
 	return &LogAggregator{
 		kubectlcli:  cli,
 		config:      config,
@@ -70,6 +70,7 @@ func NewLogAggregator(cli *kubectl.CLI, podSelector kubernetes.PodSelector, conf
 		colorPicker: kubernetes.NewColorPicker(),
 		stopWatcher: func() {},
 		events:      make(chan kubernetes.PodEvent),
+		namespaces:  namespaces,
 	}
 }
 
@@ -93,7 +94,7 @@ func (a *LogAggregator) SetSince(t time.Time) {
 
 // Start starts a logger that listens to pods and tail their logs
 // if they are matched by the `podSelector`.
-func (a *LogAggregator) Start(ctx context.Context, out io.Writer, namespaces []string) error {
+func (a *LogAggregator) Start(ctx context.Context, out io.Writer) error {
 	if a == nil {
 		// Logs are not activated.
 		return nil
@@ -102,7 +103,7 @@ func (a *LogAggregator) Start(ctx context.Context, out io.Writer, namespaces []s
 	a.output = out
 
 	a.podWatcher.Register(a.events)
-	stopWatcher, err := a.podWatcher.Start(namespaces)
+	stopWatcher, err := a.podWatcher.Start(*a.namespaces)
 	a.stopWatcher = stopWatcher
 	if err != nil {
 		return err
@@ -184,21 +185,11 @@ func (a *LogAggregator) streamContainerLogs(ctx context.Context, pod *v1.Pod, co
 
 	headerColor := a.colorPicker.Pick(pod)
 	prefix := a.prefix(pod, container)
-	if err := a.streamRequest(ctx, headerColor, prefix, pod.Name, container.Name, tr); err != nil {
+	if err := stream.StreamRequest(ctx, a.output, headerColor, prefix, pod.Name, container.Name, make(chan bool), &a.outputLock, a.IsMuted, tr); err != nil {
 		logrus.Errorf("streaming request %s", err)
 	}
 }
 
-func (a *LogAggregator) printLogLine(text string) {
-	if !a.IsMuted() {
-		a.outputLock.Lock()
-
-		fmt.Fprint(a.output, text)
-
-		a.outputLock.Unlock()
-	}
-}
-
 func (a *LogAggregator) prefix(pod *v1.Pod, container v1.ContainerStatus) string {
 	var c latestV1.Pipeline
 	var present bool
@@ -242,30 +233,6 @@ func podAndContainerPrefix(pod *v1.Pod, container v1.ContainerStatus) string {
 	return fmt.Sprintf("[%s %s]", pod.Name, container.Name)
 }
 
-func (a *LogAggregator) streamRequest(ctx context.Context, headerColor output.Color, prefix, podName, containerName string, rc io.Reader) error {
-	r := bufio.NewReader(rc)
-	for {
-		select {
-		case <-ctx.Done():
-			logrus.Infof("%s interrupted", prefix)
-			return nil
-		default:
-			// Read up to newline
-			line, err := r.ReadString('\n')
-			if err == io.EOF {
-				return nil
-			}
-			if err != nil {
-				return fmt.Errorf("reading bytes from log stream: %w", err)
-			}
-
-			formattedLine := headerColor.Sprintf("%s ", prefix) + line
-			a.printLogLine(formattedLine)
-			eventV2.ApplicationLog(podName, containerName, line, formattedLine)
-		}
-	}
-}
-
 // Mute mutes the logs.
 func (a *LogAggregator) Mute() {
 	if a == nil {
