@@ -19,6 +19,7 @@ package portforward
 import (
 	"context"
 	"fmt"
+	"io"
 	"io/ioutil"
 	"sync"
 	"testing"
@@ -60,6 +61,8 @@ func (f *testForwarder) Terminate(pfe *portForwardEntry) {
 	f.forwardedPorts.Delete(pfe.localPort)
 }
 
+func (f *testForwarder) Start(io.Writer) {}
+
 func newTestForwarder() *testForwarder {
 	return &testForwarder{}
 }
@@ -128,10 +131,10 @@ func TestStart(t *testing.T) {
 			})
 
 			fakeForwarder := newTestForwarder()
-			entryManager := NewEntryManager(ioutil.Discard, fakeForwarder)
+			entryManager := NewEntryManager(fakeForwarder)
 
 			rf := NewServicesForwarder(entryManager, "")
-			if err := rf.Start(context.Background(), []string{"test"}); err != nil {
+			if err := rf.Start(context.Background(), ioutil.Discard, []string{"test"}); err != nil {
 				t.Fatalf("error starting resource forwarder: %v", err)
 			}
 
@@ -206,7 +209,7 @@ func TestGetCurrentEntryFunc(t *testing.T) {
 				return mockRetrieveAvailablePort(util.Loopback, map[int]struct{}{}, test.availablePorts)(addr, req, ps)
 			})
 
-			entryManager := NewEntryManager(ioutil.Discard, newTestForwarder())
+			entryManager := NewEntryManager(newTestForwarder())
 			entryManager.forwardedResources = forwardedResources{
 				resources: test.forwardedResources,
 			}
@@ -313,14 +316,14 @@ func TestUserDefinedResources(t *testing.T) {
 			})
 
 			fakeForwarder := newTestForwarder()
-			entryManager := NewEntryManager(ioutil.Discard, fakeForwarder)
+			entryManager := NewEntryManager(fakeForwarder)
 
 			util.OSEnviron = func() []string {
 				return []string{"FOO=bar"}
 			}
 
 			rf := NewUserDefinedForwarder(entryManager, test.userResources)
-			if err := rf.Start(context.Background(), test.namespaces); err != nil {
+			if err := rf.Start(context.Background(), ioutil.Discard, test.namespaces); err != nil {
 				t.Fatalf("error starting resource forwarder: %v", err)
 			}
 
