@@ -23,6 +23,8 @@ import (
 	"os"
 	"testing"
 
+	"github.com/google/go-cmp/cmp/cmpopts"
+
 	"github.com/GoogleContainerTools/skaffold/pkg/skaffold/constants"
 	eventV2 "github.com/GoogleContainerTools/skaffold/pkg/skaffold/event/v2"
 	"github.com/GoogleContainerTools/skaffold/testutil"
@@ -153,7 +155,67 @@ func TestWithEventContext(t *testing.T) {
 	for _, test := range tests {
 		testutil.Run(t, test.name, func(t *testutil.T) {
 			got := WithEventContext(test.writer, test.phase, test.subtaskID, test.origin)
-			t.CheckDeepEqual(test.expected, got)
+			t.CheckDeepEqual(test.expected, got, cmpopts.IgnoreTypes(false))
+		})
+	}
+}
+
+func TestWriteWithTimeStamps(t *testing.T) {
+	tests := []struct {
+		name        string
+		writer      func(io.Writer) io.Writer
+		expectedLen int
+	}{
+		{
+			name: "skaffold writer with color and timestamps",
+			writer: func(out io.Writer) io.Writer {
+				return skaffoldWriter{
+					MainWriter:  colorableWriter{out},
+					EventWriter: ioutil.Discard,
+					timestamps:  true,
+				}
+			},
+			expectedLen: len(TimestampFormat) + len(" \u001B[32mtesting!\u001B[0m"),
+		},
+		{
+			name: "skaffold writer with color and no timestamps",
+			writer: func(out io.Writer) io.Writer {
+				return skaffoldWriter{
+					MainWriter:  colorableWriter{out},
+					EventWriter: ioutil.Discard,
+				}
+			},
+			expectedLen: len("\u001B[32mtesting!\u001B[0m"),
+		},
+		{
+			name: "skaffold writer with timestamps and no color",
+			writer: func(out io.Writer) io.Writer {
+				return skaffoldWriter{
+					MainWriter:  out,
+					EventWriter: ioutil.Discard,
+					timestamps:  true,
+				}
+			},
+			expectedLen: len(TimestampFormat) + len(" testing!"),
+		},
+		{
+			name: "skaffold writer with no color and no timestamps",
+			writer: func(out io.Writer) io.Writer {
+				return skaffoldWriter{
+					MainWriter:  out,
+					EventWriter: ioutil.Discard,
+				}
+			},
+			expectedLen: len("testing!"),
+		},
+	}
+
+	for _, test := range tests {
+		t.Run(test.name, func(t *testing.T) {
+			var buf bytes.Buffer
+			out := test.writer(&buf)
+			Default.Fprintf(out, "testing!")
+			testutil.CheckDeepEqual(t, test.expectedLen, len(buf.String()))
 		})
 	}
 }
