@@ -29,6 +29,10 @@ import (
 // unless we really want to expose something to the network.
 const Loopback = "127.0.0.1"
 
+// Network address which represent any address. This is the default that
+// we should use when checking if port is free.
+const Any = ""
+
 type PortSet struct {
 	ports map[int]bool
 	lock  sync.Mutex
@@ -88,22 +92,22 @@ func (f *PortSet) List() []int {
 }
 
 // GetAvailablePort returns an available port that is near the requested port when possible.
-// First, check if the provided port is available on the specified address. If so, use it.
+// First, check if the provided port is available on the specified address and INADDR_ANY. If so, use it.
 // If not, check if any of the next 10 subsequent ports are available.
 // If not, check if any of ports 4503-4533 are available.
 // If not, return a random port, which hopefully won't collide with any future containers
 //
 // See https://www.iana.org/assignments/service-names-port-numbers/service-names-port-numbers.txt
-func GetAvailablePort(port int, usedPorts *PortSet) int {
+func GetAvailablePort(address string, port int, usedPorts *PortSet) int {
 	if port > 0 {
-		if getPortIfAvailable(port, usedPorts) {
+		if getPortIfAvailable(address, port, usedPorts) {
 			return port
 		}
 
 		// try the next 10 ports after the provided one
 		for i := 0; i < 10; i++ {
 			port++
-			if getPortIfAvailable(port, usedPorts) {
+			if getPortIfAvailable(address, port, usedPorts) {
 				logrus.Debugf("found open port: %d", port)
 				return port
 			}
@@ -111,13 +115,13 @@ func GetAvailablePort(port int, usedPorts *PortSet) int {
 	}
 
 	for port = 4503; port <= 4533; port++ {
-		if getPortIfAvailable(port, usedPorts) {
+		if getPortIfAvailable(address, port, usedPorts) {
 			logrus.Debugf("found open port: %d", port)
 			return port
 		}
 	}
 
-	l, err := net.Listen("tcp", ":0")
+	l, err := net.Listen("tcp", fmt.Sprintf("%s:0", address))
 	if err != nil {
 		return -1
 	}
@@ -129,20 +133,29 @@ func GetAvailablePort(port int, usedPorts *PortSet) int {
 	return p
 }
 
-func getPortIfAvailable(p int, usedPorts *PortSet) bool {
+func getPortIfAvailable(address string, p int, usedPorts *PortSet) bool {
 	if alreadySet := usedPorts.LoadOrSet(p); alreadySet {
 		return false
 	}
 
-	return IsPortFree(p)
+	return IsPortFree(address, p)
 }
 
-func IsPortFree(p int) bool {
+func IsPortFree(address string, p int) bool {
+	// Ensure the port is available across all interfaces
 	l, err := net.Listen("tcp", fmt.Sprintf(":%d", p))
-	if err != nil {
+	if err != nil || l == nil {
 		return false
 	}
-
 	l.Close()
+
+	if address != Any {
+		// Ensure the port is available on the specific interface too
+		l, err := net.Listen("tcp", fmt.Sprintf("%s:%d", address, p))
+		if err != nil || l == nil {
+			return false
+		}
+		l.Close()
+	}
 	return true
 }
